import React, { Component, Fragment } from 'react';
import PropTypes from 'prop-types';
import { isNil } from 'ramda';
import { FormattedMessage } from 'react-intl';
import Button from 'reactstrap/lib/Button';

import { DotDivider } from '../../dot-divider/dot-divider';
import { IconHelp } from '../../icons/icon-help/icon-help';
import { CoachMarkPopover } from '../../coach-mark-popover/coach-mark-popover';

import './balance-item-with-hint.scss';

export const SCR = 'SCR';
export const SP = 'SP';
export const VP = 'VP';

export class BalanceItemWithHint extends Component {
  static propTypes = {
    value: PropTypes.string.isRequired,
    target: PropTypes.string.isRequired,
    titleId: PropTypes.string,
    type: PropTypes.string.isRequired,
    helpTitleId: PropTypes.string.isRequired,
    helpDescId: PropTypes.string.isRequired,
    children: PropTypes.node.isRequired,
    icon: PropTypes.node.isRequired,
    iconHelp: PropTypes.node.isRequired,
    disabledCoachMarks: PropTypes.bool,
    isOpenCoachmark: PropTypes.bool,
    toggleCoachMarkPopover: PropTypes.func,
    closeAllCoachMarks: PropTypes.func,
    popoverOpenSCR: PropTypes.bool,
    popoverOpenSP: PropTypes.bool,
    popoverOpenVP: PropTypes.bool,
    coachMarkBtnValue: PropTypes.string,
    coachMarkBtnIntlId: PropTypes.string,
    startCoach: PropTypes.func,
  };

  static defaultProps = {
    titleId: '',
    closeAllCoachMarks: null,
    toggleCoachMarkPopover: null,
    startCoach: null,
    disabledCoachMarks: true,
    isOpenCoachmark: false,
    popoverOpenSP: false,
    popoverOpenSCR: false,
    popoverOpenVP: false,
    coachMarkBtnValue: '',
    coachMarkBtnIntlId: '',
  };

  state = {
    popoverOpen: false,
    popoverHelperOpen: false,
  };

  balanceItemRef = React.createRef();

  componentDidMount() {
    const { type, startCoach } = this.props;

    if (type === SCR && !isNil(startCoach)) {
      startCoach();
    }
  }

  togglePopover = () => {
    this.setState(prevState => ({
      popoverOpen: !prevState.popoverOpen,
      popoverHelperOpen: false,
    }));
  };

  handleBalanceItemHover = (e) => {
    const { popoverOpen, popoverHelperOpen } = this.state;

    if (!popoverOpen && !popoverHelperOpen) {
      e.preventDefault();
      this.balanceItemRef.current.click();
    }
  };

  toggleHelpPopover = () =>
    this.setState(prevState => ({ popoverHelperOpen: !prevState.popoverHelperOpen, popoverOpen: false }));

  renderContent = ({ icon, helpTitleId, descriptionId }) => (
    <div className="d-flex">
      <span className="mr-1_5">{icon}</span>
      <div>
        <p className="h5 mb-0_5">
          <FormattedMessage id={helpTitleId} />
        </p>
        <p className="mb-1_5 text-small">
          <FormattedMessage id={descriptionId} />
        </p>
      </div>
    </div>
  );

  renderDotsNavigation = () => {
    const { popoverOpenSCR, popoverOpenSP, popoverOpenVP } = this.props;

    const checkColor = isNotDisabled => (isNotDisabled ? 'primary' : 'shade-s30');
    const checkLength = isActive => (isActive ? 8 : 6);
    return (
      <div className="header-balance">
        <Button color="link" className="m-0_5 p-0" value={SCR} onClick={this.props.toggleCoachMarkPopover}>
          <DotDivider
            className="d-inline-flex rounded"
            color={checkColor(popoverOpenSCR || popoverOpenSP || popoverOpenVP)}
            length={checkLength(popoverOpenSCR)}
          />
        </Button>
        <Button
          color="link"
          className="m-0_5 p-0"
          disabled={popoverOpenSCR}
          value={SP}
          onClick={this.props.toggleCoachMarkPopover}
        >
          <DotDivider
            className="d-inline-flex rounded"
            color={checkColor(popoverOpenSP || popoverOpenVP)}
            length={checkLength(popoverOpenSP)}
          />
        </Button>
        <Button
          color="link"
          className="m-0_5 p-0"
          disabled={popoverOpenSCR || popoverOpenSP}
          value={VP}
          onClick={this.props.toggleCoachMarkPopover}
        >
          <DotDivider
            className="d-inline-flex rounded"
            color={checkColor(popoverOpenVP)}
            length={checkLength(popoverOpenVP)}
          />
        </Button>
      </div>
    );
  };

  renderBlockNavigation = () => (
    <div className="d-flex justify-content-between coachmark-bottom">
      {this.renderDotsNavigation()}
      <Button
        value={this.props.coachMarkBtnValue}
        onClick={this.props.toggleCoachMarkPopover}
        color="link"
        className="text-primary text-uppercase p-0 label"
      >
        <FormattedMessage id={this.props.coachMarkBtnIntlId} />
      </Button>
    </div>
  );

  render() {
    const {
      value,
      target,
      type,
      children,
      icon,
      iconHelp,
      helpTitleId,
      helpDescId,
      titleId,
      disabledCoachMarks,
      closeAllCoachMarks,
      isOpenCoachmark,
    } = this.props;

    return (
      <Fragment>
        <FormattedMessage id={helpDescId}>
          {text => (
            <span
              className="balance-item-icon d-flex"
              onClick={this.togglePopover}
              onKeyPress={this.togglePopover}
              onMouseEnter={this.handleBalanceItemHover}
              onFocus={this.handleBalanceItemHover}
              role="button"
              tabIndex="0"
              title={text}
              ref={this.balanceItemRef}
            >
              <span id={target}>{icon}</span>
              <span className="balance h6 text-white mr-2_5 ml-1">{value}</span>
            </span>
          )}
        </FormattedMessage>

        <CoachMarkPopover
          className="header-balance-hint balance-popover-icon"
          target={target}
          isOpen={this.state.popoverOpen}
          toggle={this.togglePopover}
        >
          <span
            className="balance-item-icon"
            onClick={this.toggleHelpPopover}
            onKeyPress={this.toggleHelpPopover}
            role="button"
            tabIndex="0"
          >
            <IconHelp className="float-right mt-0_5" />
          </span>

          <FormattedMessage id={titleId || 'blog.header.balance.current'} values={{ type }}>
            {textBalance => <div className="text-small">{textBalance}</div>}
          </FormattedMessage>
          {children}
        </CoachMarkPopover>

        <CoachMarkPopover
          className="balance-item-coachmark balance-popover-icon"
          target={target}
          isOpen={this.state.popoverHelperOpen}
          toggle={this.toggleHelpPopover}
        >
          {this.renderContent({
            icon: iconHelp,
            helpTitleId,
            descriptionId: helpDescId,
          })}
          <Button
            onClick={this.toggleHelpPopover}
            color="link"
            className="text-primary text-uppercase pb-2 pt-0 float-right label pr-0"
          >
            <FormattedMessage id="blog.header.coach-mark.button.got-it" />
          </Button>
        </CoachMarkPopover>

        {!disabledCoachMarks && (
          <CoachMarkPopover
            isOpen={isOpenCoachmark}
            toggle={closeAllCoachMarks}
            target={target}
            className="balance-item-coachmark balance-popover-icon"
          >
            {this.renderContent({
              icon: iconHelp,
              helpTitleId,
              descriptionId: helpDescId,
            })}
            {this.renderBlockNavigation()}
          </CoachMarkPopover>
        )}
      </Fragment>
    );
  }
}
